@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Literal, cast, overload
+from typing import TYPE_CHECKING, Any, Callable, Literal, cast, overload
 
 import pyarrow as pa
 import pyarrow.compute as pc
@@ -15,6 +15,7 @@ from narwhals._arrow.utils import (
     chunked_array,
     extract_native,
     floordiv_compat,
+    is_array_or_scalar,
     lit,
     narwhals_to_native_dtype,
     native_to_narwhals_dtype,
@@ -23,7 +24,6 @@ from narwhals._arrow.utils import (
     zeros,
 )
 from narwhals._compliant import EagerSeries, EagerSeriesHist
-from narwhals._expression_parsing import ExprKind
 from narwhals._typing_compat import assert_never
 from narwhals._utils import (
     Implementation,
@@ -65,13 +65,12 @@ if TYPE_CHECKING:
         FillNullStrategy,
         Into1DArray,
         IntoDType,
+        ModeKeepStrategy,
         NonNestedLiteral,
-        NumericLiteral,
         PythonLiteral,
         RankMethod,
         RollingInterpolationMethod,
         SizedMultiIndexSelector,
-        TemporalLiteral,
         _1DArray,
         _2DArray,
         _SliceIndex,
@@ -146,6 +145,16 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
             result._broadcast = self._broadcast
         return result
 
+    def _with_binary(self, op: Callable[..., ArrayOrScalar], other: Any) -> Self:
+        ser, other_native = extract_native(self, other)
+        preserve_broadcast = self._broadcast and getattr(other, "_broadcast", True)
+        return self._with_native(
+            op(ser, other_native), preserve_broadcast=preserve_broadcast
+        ).alias(self.name)
+
+    def _with_binary_right(self, op: Callable[..., ArrayOrScalar], other: Any) -> Self:
+        return self._with_binary(lambda x, y: op(y, x), other).alias(self.name)
+
     @classmethod
     def from_iterable(
         cls,
@@ -156,10 +165,15 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
         dtype: IntoDType | None = None,
     ) -> Self:
         version = context._version
-        dtype_pa = narwhals_to_native_dtype(dtype, version) if dtype else None
-        return cls.from_native(
-            chunked_array([data], dtype_pa), name=name, context=context
-        )
+        if dtype is not None:
+            dtype_pa: pa.DataType | None = narwhals_to_native_dtype(dtype, version)
+            if is_array_or_scalar(data):
+                data = data.cast(dtype_pa)
+                dtype_pa = None
+            native = data if cls._is_native(data) else chunked_array([data], dtype_pa)
+        else:
+            native = chunked_array([data])
+        return cls.from_native(native, context=context, name=name)
 
     def _from_scalar(self, value: Any) -> Self:
         if hasattr(value, "as_py"):
@@ -185,16 +199,18 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
     @classmethod
     def _align_full_broadcast(cls, *series: Self) -> Sequence[Self]:
         lengths = [len(s) for s in series]
-        max_length = max(lengths)
-        fast_path = all(_len == max_length for _len in lengths)
+        target_length = max(
+            length for length, s in zip(lengths, series) if not s._broadcast
+        )
+        fast_path = all(_len == target_length for _len in lengths)
         if fast_path:
             return series
         reshaped = []
         for s in series:
             if s._broadcast:
-                compliant = s._with_native(pa.repeat(s.native[0], max_length))
-            elif (actual_len := len(s)) != max_length:
-                msg = f"Expected object of length {max_length}, got {actual_len}."
+                compliant = s._with_native(pa.repeat(s.native[0], target_length))
+            elif (actual_len := len(s)) != target_length:
+                msg = f"Expected object of length {target_length}, got {actual_len}."
                 raise ShapeError(msg)
             else:
                 compliant = s
@@ -207,106 +223,89 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
         return ArrowNamespace(version=self._version)
 
     def __eq__(self, other: object) -> Self:  # type: ignore[override]
-        other = cast("PythonLiteral | ArrowSeries | None", other)
-        ser, rhs = extract_native(self, other)
-        return self._with_native(pc.equal(ser, rhs))
+        return self._with_binary(pc.equal, other)
 
     def __ne__(self, other: object) -> Self:  # type: ignore[override]
-        other = cast("PythonLiteral | ArrowSeries | None", other)
-        ser, rhs = extract_native(self, other)
-        return self._with_native(pc.not_equal(ser, rhs))
+        return self._with_binary(pc.not_equal, other)
 
     def __ge__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.greater_equal(ser, other))
+        return self._with_binary(pc.greater_equal, other)
 
     def __gt__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.greater(ser, other))
+        return self._with_binary(pc.greater, other)
 
     def __le__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.less_equal(ser, other))
+        return self._with_binary(pc.less_equal, other)
 
     def __lt__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.less(ser, other))
+        return self._with_binary(pc.less, other)
 
     def __and__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.and_kleene(ser, other))  # type: ignore[arg-type]
+        return self._with_binary(pc.and_kleene, other)
 
     def __rand__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.and_kleene(other, ser))  # type: ignore[arg-type]
+        return self._with_binary_right(pc.and_kleene, other)
 
     def __or__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.or_kleene(ser, other))  # type: ignore[arg-type]
+        return self._with_binary_right(pc.or_kleene, other)
 
     def __ror__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.or_kleene(other, ser))  # type: ignore[arg-type]
+        return self._with_binary_right(pc.or_kleene, other)
 
     def __add__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.add(ser, other))
+        return self._with_binary(pc.add, other)
 
     def __radd__(self, other: Any) -> Self:
-        return self + other
+        return self._with_binary_right(pc.add, other)
 
     def __sub__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.subtract(ser, other))
+        return self._with_binary(pc.subtract, other)
 
     def __rsub__(self, other: Any) -> Self:
-        return (self - other) * (-1)
+        return self._with_binary_right(pc.subtract, other)
 
     def __mul__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.multiply(ser, other))
+        return self._with_binary(pc.multiply, other)
 
     def __rmul__(self, other: Any) -> Self:
-        return self * other
+        return self._with_binary_right(pc.multiply, other)
 
     def __pow__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.power(ser, other))
+        return self._with_binary(pc.power, other)
 
     def __rpow__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.power(other, ser))
+        return self._with_binary_right(pc.power, other)
 
     def __floordiv__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(floordiv_compat(ser, other))
+        return self._with_binary(floordiv_compat, other)
 
     def __rfloordiv__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(floordiv_compat(other, ser))
+        return self._with_binary_right(floordiv_compat, other)
 
     def __truediv__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.divide(*cast_for_truediv(ser, other)))  # type: ignore[type-var]
+        return self._with_binary(lambda x, y: pc.divide(*cast_for_truediv(x, y)), other)
 
     def __rtruediv__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.divide(*cast_for_truediv(other, ser)))  # type: ignore[type-var]
+        return self._with_binary_right(
+            lambda x, y: pc.divide(*cast_for_truediv(x, y)), other
+        )
 
     def __mod__(self, other: Any) -> Self:
+        preserve_broadcast = self._broadcast and getattr(other, "_broadcast", True)
         floor_div = (self // other).native
         ser, other = extract_native(self, other)
         res = pc.subtract(ser, pc.multiply(floor_div, other))
-        return self._with_native(res)
+        return self._with_native(res, preserve_broadcast=preserve_broadcast)
 
     def __rmod__(self, other: Any) -> Self:
+        preserve_broadcast = self._broadcast and getattr(other, "_broadcast", True)
         floor_div = (other // self).native
         ser, other = extract_native(self, other)
         res = pc.subtract(other, pc.multiply(floor_div, ser))
-        return self._with_native(res)
+        return self._with_native(res, preserve_broadcast=preserve_broadcast)
 
     def __invert__(self) -> Self:
-        return self._with_native(pc.invert(self.native))
+        return self._with_native(pc.invert(self.native), preserve_broadcast=True)
 
     @property
     def _type(self) -> pa.DataType:
@@ -323,6 +322,15 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
             other_native = predicate
         return self._with_native(self.native.filter(other_native))
 
+    def first(self, *, _return_py_scalar: bool = True) -> PythonLiteral:
+        result = self.native[0] if len(self.native) else None
+        return maybe_extract_py_scalar(result, _return_py_scalar)
+
+    def last(self, *, _return_py_scalar: bool = True) -> PythonLiteral:
+        ca = self.native
+        result = ca[height - 1] if (height := len(ca)) else None
+        return maybe_extract_py_scalar(result, _return_py_scalar)
+
     def mean(self, *, _return_py_scalar: bool = True) -> float:
         return maybe_extract_py_scalar(pc.mean(self.native), _return_py_scalar)
 
@@ -366,12 +374,12 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
             return self._with_native(self.native)
         return self._with_native(pa.concat_arrays(arrays))
 
-    def std(self, ddof: int, *, _return_py_scalar: bool = True) -> float:
+    def std(self, *, ddof: int, _return_py_scalar: bool = True) -> float:
         return maybe_extract_py_scalar(
             pc.stddev(self.native, ddof=ddof), _return_py_scalar
         )
 
-    def var(self, ddof: int, *, _return_py_scalar: bool = True) -> float:
+    def var(self, *, ddof: int, _return_py_scalar: bool = True) -> float:
         return maybe_extract_py_scalar(
             pc.variance(self.native, ddof=ddof), _return_py_scalar
         )
@@ -482,9 +490,9 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
         return self.native.to_numpy()
 
     def alias(self, name: str) -> Self:
-        result = self.__class__(self.native, name=name, version=self._version)
-        result._broadcast = self._broadcast
-        return result
+        ret = self.__class__(self.native, name=name, version=self._version)
+        ret._broadcast = self._broadcast
+        return ret
 
     @property
     def dtype(self) -> DType:
@@ -507,6 +515,12 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
             pc.round(self.native, decimals, round_mode="half_towards_infinity")
         )
 
+    def floor(self) -> Self:
+        return self._with_native(pc.floor(self.native))
+
+    def ceil(self) -> Self:
+        return self._with_native(pc.ceil(self.native))
+
     def diff(self) -> Self:
         return self._with_native(pc.pairwise_diff(self.native.combine_chunks()))
 
@@ -644,6 +658,10 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
         mask = rng.choice(idx, size=n, replace=with_replacement)
         return self._with_native(self.native.take(mask))
 
+    def fill_nan(self, value: float | None) -> Self:
+        result = pc.if_else(pc.is_nan(self.native), value, self.native)
+        return self._with_native(result, preserve_broadcast=True)
+
     def fill_null(
         self,
         value: Self | NonNestedLiteral,
@@ -745,7 +763,7 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
             result = pc.all(pc.less_equal(self.native[:-1], self.native[1:]))
         return maybe_extract_py_scalar(result, return_py_scalar=True)
 
-    def unique(self, *, maintain_order: bool) -> Self:
+    def unique(self, *, maintain_order: bool = True) -> Self:
         # TODO(marco): `pc.unique` seems to always maintain order, is that guaranteed?
         return self._with_native(self.native.unique())
 
@@ -827,39 +845,34 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
     def gather_every(self, n: int, offset: int = 0) -> Self:
         return self._with_native(self.native[offset::n])
 
-    def clip(
-        self,
-        lower_bound: Self | NumericLiteral | TemporalLiteral | None,
-        upper_bound: Self | NumericLiteral | TemporalLiteral | None,
-    ) -> Self:
-        _, lower = (
-            extract_native(self, lower_bound) if lower_bound is not None else (None, None)
-        )
-        _, upper = (
-            extract_native(self, upper_bound) if upper_bound is not None else (None, None)
-        )
-
-        if lower is None:
-            return self._with_native(pc.min_element_wise(self.native, upper))
-        if upper is None:
-            return self._with_native(pc.max_element_wise(self.native, lower))
+    def clip(self, lower_bound: Self, upper_bound: Self) -> Self:
+        _, lower = extract_native(self, lower_bound)
+        _, upper = extract_native(self, upper_bound)
         return self._with_native(
             pc.max_element_wise(pc.min_element_wise(self.native, upper), lower)
         )
 
+    def clip_lower(self, lower_bound: Self) -> Self:
+        _, lower = extract_native(self, lower_bound)
+        return self._with_native(pc.max_element_wise(self.native, lower))
+
+    def clip_upper(self, upper_bound: Self) -> Self:
+        _, upper = extract_native(self, upper_bound)
+        return self._with_native(pc.min_element_wise(self.native, upper))
+
     def to_arrow(self) -> ArrayAny:
         return self.native.combine_chunks()
 
-    def mode(self) -> ArrowSeries:
+    def mode(self, *, keep: ModeKeepStrategy) -> ArrowSeries:
         plx = self.__narwhals_namespace__()
         col_token = generate_temporary_column_name(n_bytes=8, columns=[self.name])
         counts = self.value_counts(
             name=col_token, normalize=False, sort=False, parallel=False
         )
-        return counts.filter(
-            plx.col(col_token)
-            == plx.col(col_token).max().broadcast(kind=ExprKind.AGGREGATION)
+        result = counts.filter(
+            plx.col(col_token) == plx.col(col_token).max().broadcast()
         ).get_column(self.name)
+        return result.head(1) if keep == "any" else result
 
     def is_finite(self) -> Self:
         return self._with_native(pc.is_finite(self.native))
