@@ -3,7 +3,6 @@ from __future__ import annotations
 from datetime import datetime, time, timedelta, timezone
 from typing import TYPE_CHECKING, cast
 
-import pandas as pd
 import pytest
 
 import narwhals as nw
@@ -18,7 +17,10 @@ from tests.utils import (
 )
 
 if TYPE_CHECKING:
+    from collections.abc import Mapping
+
     from narwhals._native import NativeSQLFrame
+    from narwhals.typing import NonNestedDType
 
 DATA = {
     "a": [1],
@@ -38,7 +40,7 @@ DATA = {
     "o": ["a"],
     "p": [1],
 }
-SCHEMA = {
+SCHEMA: Mapping[str, type[NonNestedDType]] = {
     "a": nw.Int64,
     "b": nw.Int32,
     "c": nw.Int16,
@@ -86,7 +88,7 @@ def test_cast(constructor: Constructor) -> None:
         nw.col(col_).cast(dtype) for col_, dtype in schema.items()
     )
 
-    cast_map = {
+    cast_map: Mapping[str, type[NonNestedDType]] = {
         "a": nw.Int32,
         "b": nw.Int16,
         "c": nw.Int8,
@@ -134,7 +136,7 @@ def test_cast_series(
         .lazy()
         .collect()
     )
-    cast_map = {
+    cast_map: Mapping[str, type[NonNestedDType]] = {
         "a": nw.Int32,
         "b": nw.Int16,
         "c": nw.Int8,
@@ -165,6 +167,9 @@ def test_cast_series(
 
 
 def test_cast_string() -> None:
+    pytest.importorskip("pandas")
+    import pandas as pd
+
     s_pd = pd.Series([1, 2]).convert_dtypes()
     s = nw.from_native(s_pd, series_only=True)
     s = s.cast(nw.String)
@@ -408,7 +413,10 @@ def test_cast_typing_invalid() -> None:
 
 @pytest.mark.skipif(PANDAS_VERSION < (2,), reason="too old for pyarrow")
 def test_pandas_pyarrow_dtypes() -> None:
+    pytest.importorskip("pandas")
     pytest.importorskip("pyarrow")
+    import pandas as pd
+
     s = nw.from_native(
         pd.Series([123, None]).convert_dtypes(dtype_backend="pyarrow"), series_only=True
     ).cast(nw.String)
@@ -428,6 +436,9 @@ def test_pandas_pyarrow_dtypes() -> None:
 
 
 def test_cast_object_pandas() -> None:
+    pytest.importorskip("pandas")
+    import pandas as pd
+
     s = nw.from_native(pd.DataFrame({"a": [2, 3, None]}, dtype=object))["a"]
     assert s[0] == 2
     assert s.cast(nw.String)[0] == "2"
