@@ -6,7 +6,6 @@ from random import Random
 from typing import TYPE_CHECKING, Any
 
 import hypothesis.strategies as st
-import pandas as pd
 import pytest
 from hypothesis import given
 
@@ -134,6 +133,9 @@ def test_hist_count(
     library: str, *, params: dict[str, Any], include_breakpoint: bool
 ) -> None:
     if library == "pandas":
+        pytest.importorskip("pandas")
+        import pandas as pd
+
         constructor_eager: Any = pd.DataFrame
     elif library == "polars":
         pl = pytest.importorskip("polars")
@@ -184,6 +186,9 @@ def test_hist_count(
 @param_library
 def test_hist_count_no_spread(library: str) -> None:
     if library == "pandas":
+        pytest.importorskip("pandas")
+        import pandas as pd
+
         constructor_eager: Any = pd.DataFrame
     elif library == "polars":
         pl = pytest.importorskip("polars")
@@ -223,6 +228,9 @@ def test_hist_bin_and_bin_count() -> None:
 @param_library
 def test_hist_no_data(library: str, *, include_breakpoint: bool) -> None:
     if library == "pandas":
+        pytest.importorskip("pandas")
+        import pandas as pd
+
         constructor_eager: Any = pd.DataFrame
     elif library == "polars":
         pl = pytest.importorskip("polars")
@@ -252,6 +260,9 @@ def test_hist_no_data(library: str, *, include_breakpoint: bool) -> None:
 @param_library
 def test_hist_small_bins(library: str) -> None:
     if library == "pandas":
+        pytest.importorskip("pandas")
+        import pandas as pd
+
         constructor_eager: Any = pd.DataFrame
     elif library == "polars":
         pl = pytest.importorskip("polars")
@@ -313,6 +324,9 @@ def test_hist_bin_hypotheis(
     library: str, data: list[float], bin_deltas: list[float]
 ) -> None:
     if library == "pandas":
+        pytest.importorskip("pandas")
+        import pandas as pd
+
         constructor_eager: Any = pd.DataFrame
     elif library == "polars":
         pl = pytest.importorskip("polars")
@@ -364,6 +378,9 @@ def test_hist_count_hypothesis(
     import polars as pl
 
     if library == "pandas":
+        pytest.importorskip("pandas")
+        import pandas as pd
+
         constructor_eager: Any = pd.DataFrame
     elif library == "polars":
         constructor_eager = pl.DataFrame
